=== modified file 'bzrlib/graph.py'
--- bzrlib/graph.py	2008-01-16 01:25:39 +0000
+++ bzrlib/graph.py	2008-01-17 22:41:32 +0000
@@ -660,15 +660,15 @@
             stopped = self._next_query.intersection(revisions)
             self._next_query = self._next_query.difference(revisions)
         else:
-            stopped = set()
-            stopped.update(self._current_present.intersection(revisions))
-            stopped.update(self._current_ghosts.intersection(revisions))
+            stopped_present = self._current_present.intersection(revisions)
+            stopped = stopped_present.union(
+                self._current_ghosts.intersection(revisions))
             self._current_present.difference_update(stopped)
             self._current_ghosts.difference_update(stopped)
             # stopping 'x' should stop returning parents of 'x', but 
             # not if 'y' always references those same parents
             stop_rev_references = {}
-            for rev in stopped:
+            for rev in stopped_present:
                 for parent_id in self._current_parents[rev]:
                     if parent_id not in stop_rev_references:
                         stop_rev_references[parent_id] = 0

=== modified file 'bzrlib/tests/test_graph.py'
--- bzrlib/tests/test_graph.py	2008-01-16 01:25:39 +0000
+++ bzrlib/tests/test_graph.py	2008-01-17 22:41:32 +0000
@@ -818,6 +818,30 @@
         search.start_searching(['head'])
         self.assertSeenAndResult(expected, search, search.next_with_ghosts)
 
+    def test_breadth_first_stop_searching_not_queried(self):
+        # A client should be able to say 'stop node X' even if X has not been
+        # returned to the client.
+        graph = self.make_graph({
+            'head':['child', 'ghost1'],
+            'child':[NULL_REVISION],
+            NULL_REVISION:[],
+            })
+        search = graph._make_breadth_first_searcher(['head'])
+        expected = [
+            # NULL_REVISION and ghost1 have not been returned
+            (set(['head']), (set(['head']), set(['child', 'ghost1']), 1),
+             ['head'], None, [NULL_REVISION, 'ghost1']),
+            # ghost1 has been returned, NULL_REVISION is to be returned in the
+            # next iteration.
+            (set(['head', 'child', 'ghost1']),
+             (set(['head']), set(['ghost1', NULL_REVISION]), 2),
+             ['head', 'child'], None, [NULL_REVISION, 'ghost1']),
+            ]
+        self.assertSeenAndResult(expected, search, search.next)
+        # using next_with_ghosts:
+        search = graph._make_breadth_first_searcher(['head'])
+        self.assertSeenAndResult(expected, search, search.next_with_ghosts)
+
     def test_breadth_first_get_result_ghosts_are_excluded(self):
         graph = self.make_graph({
             'head':['child', 'ghost'],

